@@ -57,6 +57,13 @@ def write_yum_repo(content, filename='ceph.repo'):
     write_file(repo_path, content)
 
 
+def set_apt_priority(fqdn, path='/etc/apt/preferences.d/ceph.pref'):
+    template = "Package: *\nPin: origin {fqdn}\nPin-Priority: 999\n"
+    content = template.format(fqdn=fqdn)
+    with open(path, 'wb') as fout:
+        fout.write(content)
+
+
 def set_repo_priority(sections, path='/etc/yum.repos.d/ceph.repo', priority='1'):
     Config = ConfigParser.ConfigParser()
     Config.read(path)
@@ -94,7 +101,7 @@ def set_repo_priority(sections, path='/etc/yum.repos.d/ceph.repo', priority='1')
 def write_conf(cluster, conf, overwrite):
     """ write cluster configuration to /etc/ceph/{cluster}.conf """
     path = '/etc/ceph/{cluster}.conf'.format(cluster=cluster)
-    tmp_file = tempfile.NamedTemporaryFile(delete=False)
+    tmp_file = tempfile.NamedTemporaryFile(dir='/etc/ceph', delete=False)
     err_msg = 'config file %s exists with different content; use --overwrite-conf to overwrite' % path
 
     if os.path.exists(path):
@@ -212,6 +219,33 @@ def get_file(path):
         pass
 
 
+def object_grep(term, file_object):
+    for line in file_object.readlines():
+        if term in line:
+            return True
+    return False
+
+
+def grep(term, file_path):
+    # A small grep-like function that will search for a word in a file and
+    # return True if it does and False if it does not.
+
+    # Implemented initially to have a similar behavior as the init system
+    # detection in Ceph's init scripts::
+
+    #     # detect systemd
+    #     # SYSTEMD=0
+    #     grep -qs systemd /proc/1/comm && SYSTEMD=1
+
+    # .. note:: Because we intent to be operating in silent mode, we explicitly
+    # return ``False`` if the file does not exist.
+    if not os.path.isfile(file_path):
+        return False
+
+    with open(file_path) as _file:
+        return object_grep(term, _file)
+
+
 def shortname():
     """get remote short hostname"""
     return socket.gethostname().split('.', 1)[0]
