@@ -8,8 +8,9 @@ import time
 
 from ceph_deploy import conf, exc, admin
 from ceph_deploy.cliutil import priority
-from ceph_deploy.util import paths, net
-from ceph_deploy.lib.remoto import process
+from ceph_deploy.util import paths, net, files
+from ceph_deploy.lib import remoto
+from ceph_deploy.new import new_mon_keyring
 from ceph_deploy import hosts
 from ceph_deploy.misc import mon_hosts
 from ceph_deploy.connection import get_connection
@@ -30,7 +31,7 @@ def mon_status_check(conn, logger, hostname, args):
     """
     asok_path = paths.mon.asok(args.cluster, hostname)
 
-    out, err, code = process.check(
+    out, err, code = remoto.process.check(
         conn,
         [
             'ceph',
@@ -99,6 +100,9 @@ def mon_status(conn, logger, hostname, args, silent=False):
         if out['rank'] >= 0:
             logger.info('monitor: %s is running' % mon)
             return True
+        if out['rank'] == -1 and out['state']:
+            logger.info('monitor: %s is currently at the state of %s' % (mon, out['state']))
+            return True
         logger.info('monitor: %s is not running' % mon)
         return False
     except RuntimeError:
@@ -106,12 +110,73 @@ def mon_status(conn, logger, hostname, args, silent=False):
         return False
 
 
+def keyring_parser(path):
+    """
+    This is a very, very, dumb parser that will look for `[entity]` sections
+    and return a list of those sections. It is not possible to parse this with
+    ConfigParser even though it is almost the same thing.
+
+    Since this is only used to spit out warnings, it is OK to just be naive
+    about the parsing.
+    """
+    sections = []
+    with open(path) as keyring:
+        lines = keyring.readlines()
+        for line in lines:
+            line = line.strip('\n')
+            if line.startswith('[') and line.endswith(']'):
+                sections.append(line.strip('[]'))
+    return sections
+
+
+def concatenate_keyrings(args):
+    """
+    A helper to collect all keyrings into a single blob that will be
+    used to inject it to mons with ``--mkfs`` on remote nodes
+
+    We require all keyring files to be concatenated to be in a directory
+    to end with ``.keyring``.
+    """
+    keyring_path = os.path.abspath(args.keyrings)
+    LOG.info('concatenating keyrings from %s' % keyring_path)
+    LOG.info('to seed remote monitors')
+
+    keyrings = [
+        os.path.join(keyring_path, f) for f in os.listdir(keyring_path)
+        if os.path.isfile(os.path.join(keyring_path, f)) and f.endswith('.keyring')
+    ]
+
+    contents = []
+    seen_sections = {}
+
+    if not keyrings:
+        path_from_arg = os.path.abspath(args.keyrings)
+        raise RuntimeError('could not find any keyrings in %s' % path_from_arg)
+
+    for keyring in keyrings:
+        path = os.path.abspath(keyring)
+
+        for section in keyring_parser(path):
+            if not seen_sections.get(section):
+                seen_sections[section] = path
+                LOG.info('adding entity "%s" from keyring %s' % (section, path))
+                with open(path) as k:
+                    contents.append(k.read())
+            else:
+                LOG.warning('will not add keyring: %s' % path)
+                LOG.warning('entity "%s" from keyring %s is a duplicate' % (section, path))
+                LOG.warning('already present in keyring: %s' % seen_sections[section])
+
+    return ''.join(contents)
+
+
 def mon_add(args):
     cfg = conf.ceph.load(args)
 
     if not args.mon:
         raise exc.NeedHostError()
     mon_host = args.mon[0]
+
     try:
         with file('{cluster}.mon.keyring'.format(cluster=args.cluster),
                   'rb') as f:
@@ -176,12 +241,16 @@ def mon_create(args):
     if not args.mon:
         raise exc.NeedHostError()
 
-    try:
-        with file('{cluster}.mon.keyring'.format(cluster=args.cluster),
-                  'rb') as f:
-            monitor_keyring = f.read()
-    except IOError:
-        raise RuntimeError('mon keyring not found; run \'new\' to create a new cluster')
+    if args.keyrings:
+        monitor_keyring = concatenate_keyrings(args)
+    else:
+        keyring_path = '{cluster}.mon.keyring'.format(cluster=args.cluster)
+        try:
+            monitor_keyring = files.read_file(keyring_path)
+        except IOError:
+            LOG.warning('keyring (%s) not found, creating a new one' % keyring_path)
+            new_mon_keyring(args)
+            monitor_keyring = files.read_file(keyring_path)
 
     LOG.debug(
         'Deploying mon, cluster %s hosts %s',
@@ -243,7 +312,7 @@ def destroy_mon(conn, cluster, hostname):
 
     if conn.remote_module.path_exists(path):
         # remove from cluster
-        process.run(
+        remoto.process.run(
             conn,
             [
                 'ceph',
@@ -292,7 +361,7 @@ def destroy_mon(conn, cluster, hostname):
             stamp=datetime.datetime.utcnow().strftime("%Y-%m-%dZ%H:%M:%S"),
             )
 
-        process.run(
+        remoto.process.run(
             conn,
             [
                 'mkdir',
@@ -446,6 +515,13 @@ def make(parser):
         dest='address',
     )
 
+    parser.add_argument(
+        '--keyrings',
+        nargs='?',
+        dest='keyrings',
+        help='concatenate multiple keyrings to be seeded on new monitors',
+    )
+
     parser.add_argument(
         'mon',
         nargs='*',
@@ -476,7 +552,7 @@ def is_running(conn, args):
         mon.mira094: dead {"version":"0.61.5"}
         mon.mira094: not running {"version":"0.61.5"}
     """
-    stdout, stderr, _ = process.check(
+    stdout, stderr, _ = remoto.process.check(
         conn,
         args
     )
